@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.82 2000/06/27 11:36:39 nisse Exp $
+ * $Id: channel.c,v 1.83 2000/08/03 23:31:36 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -1857,7 +1857,7 @@ channel_transmit_extended(struct ssh_channel *channel,
      (name channel_write_extended)
      (super channel_write)
      (vars
-       (type simple UINT32)))
+       (type . UINT32)))
 */
 
 static void
