@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.48 2000/06/04 19:51:30 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.49 2000/08/03 23:35:20 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -53,7 +53,7 @@
      (super packet_handler)
      (vars
        (dh struct dh_instance)
-       (hostkey_algorithm . UINT32)
+       (hostkey_algorithm . int)
        (verifier object lookup_verifier)
        (algorithms object object_list)))
 */
