@@ -1,6 +1,6 @@
 /* connection_commands.h
  *
- * $Id: connection_commands.h,v 1.11 2000/07/27 21:35:44 nisse Exp $
+ * $Id: connection_commands.h,v 1.12 2000/08/03 23:43:15 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -42,9 +42,9 @@
        ; CONNECTION_SERVER or CONNECTION_CLIENT
        (flags . UINT32)
        
-       (block_size simple UINT32)
-       (id_comment simple "const char *")
-       (debug_comment simple "const char *")
+       (block_size . UINT32)
+       (id_comment . "const char *")
+       (debug_comment . "const char *")
 
        (random object randomness)
        (algorithms object alist)
