@@ -1,6 +1,6 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.19 2000/06/10 13:14:14 nisse Exp $ */
+ * $Id: exception.h,v 1.20 2000/07/27 21:35:44 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -146,7 +146,7 @@ void exception_raise(struct exception_handler *e,
 #define EXC_ALL (~0)
 
 extern struct exception_handler default_exception_handler;
-struct exception_handler ignore_exception_handler;
+extern struct exception_handler ignore_exception_handler;
 extern struct exception dummy_exception;
 
 /* GABA:
