@@ -2,7 +2,7 @@
  *
  * Decode ssh2 keys.
  *
- * $Id: lsh-decode-key.c,v 1.1 2000/06/10 13:20:42 nisse Exp $
+ * $Id: lsh-decode-key.c,v 1.2 2000/08/03 23:51:26 nisse Exp $
  */
 
 #include "io.h"
@@ -139,7 +139,7 @@ do_decode_key(struct abstract_write *s,
   else
     {
       struct simple_buffer buffer;
-      UINT32 type;
+      int type;
 
       simple_buffer_init(&buffer, contents->length, contents->data);
 
