@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.114 2000/06/21 19:31:06 nisse Exp $ */
+ * $Id: lsh.c,v 1.115 2000/07/27 22:05:08 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -262,7 +262,7 @@ do_options2known_hosts(struct command *ignored UNUSED,
     s = options->known_hosts;
   else 
     {
-      tmp = ssh_format("%lz/.lsh/known_hosts%c", options->home, 0);
+      tmp = ssh_cformat("%lz/.lsh/known_hosts", options->home);
       s = tmp->data;
     }
   
@@ -310,7 +310,7 @@ do_options2identities(struct command *ignored UNUSED,
     s = options->identity;
   else 
     {
-      tmp = ssh_format("%lz/.lsh/identity%c", options->home, 0);
+      tmp = ssh_cformat("%lz/.lsh/identity", options->home);
       s = tmp->data;
     }
   
@@ -1157,7 +1157,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	    s = self->capture;
 	  else if (self->sloppy)
 	    {
-	      tmp = ssh_format("%lz/.lsh/captured_keys%c", self->home, 0);
+	      tmp = ssh_cformat("%lz/.lsh/captured_keys", self->home);
 	      s = tmp->data;
 	    }
 	  if (s)
