@@ -2,7 +2,7 @@
  *
  * Utility macros for managing a struct simple_buffer *buffer.
  *
- * $Id: parse_macros.h,v 1.1 1999/01/02 06:01:29 nisse Exp $ */
+ * $Id: parse_macros.h,v 1.2 2000/06/27 11:36:40 nisse Exp $ */
 
 #ifndef LSH_PARSE_MACROS_H_INCLUDED
 #define LSH_PARSE_MACROS_H_INCLUDED
@@ -16,4 +16,4 @@
 #define ADVANCE(n) (BUFFER->pos  += (n))
 #define GET() (BUFFER->data[BUFFER->pos++])
 
-#endif */ LSH_PARSE_MACROS_H_INCLUDED */
+#endif /* LSH_PARSE_MACROS_H_INCLUDED */
