@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.47 2000/06/27 20:47:47 nisse Exp $
+ * $Id: server_session.c,v 1.48 2000/07/27 22:05:09 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -812,7 +812,8 @@ do_spawn_exec(struct channel_request *c,
     /* Already spawned a shell or command */
     goto fail;
 
-  command_line = make_cstring_l(command_len, command);
+  command_line = ssh_cformat("%ls", command_len, command);
+  
   if (!command_line)
     EXCEPTION_RAISE(e, &exec_request_failed);
   else
