@@ -2,7 +2,7 @@
  *
  * An implementation of SPKI certificate checking
  *
- * $Id: spki.c,v 1.20 2000/06/01 13:18:11 nisse Exp $ */
+ * $Id: spki.c,v 1.21 2000/08/04 00:20:21 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -124,7 +124,7 @@ dsa_to_spki_public_key(struct dsa_public *dsa)
 int spki_get_type(struct sexp *e, struct sexp_iterator **res)
 {
   struct sexp_iterator *i;
-  UINT32 type;
+  int type;
   
   if (sexp_atomp(e) || sexp_nullp(e))
     return 0;
