@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.22 2000/07/27 22:05:10 nisse Exp $
+ * $Id: unix_user.c,v 1.23 2000/08/04 00:17:21 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -221,7 +221,7 @@ make_process_resource(pid_t pid, int signal)
      (name unix_user)
      (super lsh_user)
      (vars
-       (gid simple gid_t)
+       (gid . gid_t)
        ; Needed for the USER_READ_FILE method
        (backend object io_backend)
        
